@@ -429,7 +429,7 @@ static struct PgqTableInfo *find_table_info(Relation rel)
 
 static struct PgqTriggerInfo *find_trigger_info(struct PgqTableInfo *info, Oid tgoid, bool create)
 {
-	struct PgqTriggerInfo *tgargs = info->tg_cache;
+	struct PgqTriggerInfo *tgargs;
 	for (tgargs = info->tg_cache; tgargs; tgargs = tgargs->next) {
 		if (tgargs->tgoid == tgoid)
 			return tgargs;
@@ -494,7 +494,7 @@ static void parse_oldstyle_args(PgqTriggerEvent *ev, TriggerData *tg)
 {
 	const char *kpos;
 	int attcnt, i;
-	TupleDesc tupdesc = tg->tg_relation->rd_att;
+	TupleDesc tupdesc;
 
 	if (tg->tg_trigger->tgnargs < 2 || tg->tg_trigger->tgnargs > 3)
 		elog(ERROR, "pgq.logtriga must be used with 2 or 3 args");
@@ -525,7 +525,7 @@ static void parse_oldstyle_args(PgqTriggerEvent *ev, TriggerData *tg)
 /*
  * parse trigger arguments.
  */
-void pgq_prepare_event(struct PgqTriggerEvent *ev, TriggerData *tg, bool newstyle)
+void pgq_prepare_event(struct PgqTriggerEvent *ev, TriggerData *tg, bool newstyle, bool jsonbackup)
 {
 	memset(ev, 0, sizeof(*ev));
 
@@ -614,9 +614,13 @@ void pgq_prepare_event(struct PgqTriggerEvent *ev, TriggerData *tg, bool newstyl
 	/*
 	 * Do the backup, if requested.
 	 */
-	if (ev->tgargs->backup) {
+	if (ev->tgargs->backup && ev->op_type == 'U') {
 		ev->field[EV_EXTRA2] = pgq_init_varbuf();
-		pgq_urlenc_row(ev, tg->tg_trigtuple, ev->field[EV_EXTRA2]);
+		if (jsonbackup) {
+			pgq_jsonenc_row(ev, tg->tg_trigtuple, ev->field[EV_EXTRA2]);
+		} else {
+			pgq_urlenc_row(ev, tg->tg_trigtuple, ev->field[EV_EXTRA2]);
+		}
 	}
 }
 
@@ -808,13 +812,13 @@ static void override_fields(struct PgqTriggerEvent *ev)
 		if (i == EV_WHEN) {
 			bool isnull;
 			Oid oid = SPI_gettypeid(SPI_tuptable->tupdesc, 1);
-			Datum res;
+			Datum when_res;
 			if (oid != BOOLOID)
 				elog(ERROR, "when= query result must be boolean, got=%u", oid);
-			res = SPI_getbinval(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1, &isnull);
+			when_res = SPI_getbinval(SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1, &isnull);
 			if (isnull)
 				elog(ERROR, "when= should not be NULL");
-			if (DatumGetBool(res) == 0)
+			if (DatumGetBool(when_res) == 0)
 				ev->skip_event = true;
 			continue;
 		}
